
/* Chrysalide - Outil d'analyse de fichiers binaires
 * move.c - décompilation des opérations de déplacement
 *
 * Copyright (C) 2012-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "translate.h"


#include "../../../decomp/expr/assign.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : instr = instruction d'origine à convertir.                   *
*                ctx   = contexte de la phase de décompilation.               *
*                                                                             *
*  Description : Décompile une instruction de type 'move'.                    *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDecInstruction *dalvik_decomp_instr_move(const GArchInstruction *instr, GDecContext *ctx)
{
    GDecInstruction *result;                /* Instruction à retourner     */
    vmpa_t addr;                            /* Adresse de l'instruction    */
    GArchOperand *operand;                  /* Opérande de l'instruction   */
    GDecInstruction *src;                   /* Registre de l'object        */
    GDecInstruction *dest;                  /* Registre de destination     */

    g_arch_instruction_get_location(instr, NULL, NULL, &addr);

    operand = g_arch_instruction_get_operand(instr, 1);
    src = g_dec_context_convert_register(ctx, operand, false, addr);

    operand = g_arch_instruction_get_operand(instr, 0);
    dest = g_dec_context_convert_register(ctx, operand, true, addr);

    result = g_assign_expression_new(G_DEC_EXPRESSION(dest), G_DEC_EXPRESSION(src));

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : instr = instruction d'origine à convertir.                   *
*                ctx   = contexte de la phase de décompilation.               *
*                                                                             *
*  Description : Décompile une instruction de type 'move-object'.             *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDecInstruction *dalvik_decomp_instr_move_object(const GArchInstruction *instr, GDecContext *ctx)
{
    GDecInstruction *result;                /* Instruction à retourner     */
    vmpa_t addr;                            /* Adresse de l'instruction    */
    GArchOperand *operand;                  /* Opérande de l'instruction   */
    GDecInstruction *src;                   /* Registre de l'object        */
    GDecInstruction *dest;                  /* Registre de destination     */

    g_arch_instruction_get_location(instr, NULL, NULL, &addr);

    operand = g_arch_instruction_get_operand(instr, 1);
    src = g_dec_context_convert_register(ctx, operand, false, addr);

    operand = g_arch_instruction_get_operand(instr, 0);
    dest = g_dec_context_convert_register(ctx, operand, true, addr);

    result = g_assign_expression_new(G_DEC_EXPRESSION(dest), G_DEC_EXPRESSION(src));

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : instr = instruction d'origine à convertir.                   *
*                ctx   = contexte de la phase de décompilation.               *
*                                                                             *
*  Description : Décompile une instruction de type 'move-result'.             *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDecInstruction *dalvik_decomp_instr_move_result(const GArchInstruction *instr, GDecContext *ctx)
{
    GDecInstruction *result;                /* Instruction à retourner     */
    GDecInstruction *list;                  /* Instructions décompilées    */
    vmpa_t addr;                            /* Adresse de l'instruction    */
    GArchOperand *operand;                  /* Opérande de l'instruction   */
    GDecInstruction *last;                  /* Instruction précédante      */
    GDecInstruction *dest;                  /* Registre de destination     */

    list = g_dec_context_get_decomp_instrs(ctx);
    if (list == NULL) return NULL;

    last = g_dec_instruction_get_last(list);
    g_dec_instruction_delete(&list, last);
    g_dec_context_set_decomp_instrs(ctx, list);

    g_arch_instruction_get_location(instr, NULL, NULL, &addr);

    operand = g_arch_instruction_get_operand(instr, 0);
    dest = g_dec_context_convert_register(ctx, operand, true, addr);

    result = g_assign_expression_new(G_DEC_EXPRESSION(dest), G_DEC_EXPRESSION(last));

    return result;

}
